"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from .addressvalidationresults import AddressValidationResults
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from shippo import utils
from typing import Optional, Union


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Address:
    r"""Address represents the address as retrieved from the database"""
    country: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('country') }})
    r"""ISO 3166-1 alpha-2 country codes and country names can be used. For most consistent results, we reccomend using country codes like `US` or `DE`.
    If using country names, please ensure they are spelled correctly and in English. Country names are converted to country codes.
    Refer to this <a href=\"https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2#Officially_assigned_code_elements\" target=\"_blank\">guide</a> for a list of country codes.
    Sending a country is always required.
    """
    name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name'), 'exclude': lambda f: f is None }})
    r"""**required for purchase**<br>
    First and Last Name of the addressee
    """
    company: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('company'), 'exclude': lambda f: f is None }})
    r"""Company Name"""
    street1: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('street1'), 'exclude': lambda f: f is None }})
    r"""**required for purchase**<br>
    First street line, 35 character limit. Usually street number and street name (except for DHL Germany, see street_no).
    """
    street2: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('street2'), 'exclude': lambda f: f is None }})
    r"""Second street line, 35 character limit."""
    street3: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('street3'), 'exclude': lambda f: f is None }})
    r"""Third street line, 35 character limit.
    Only accepted for USPS international shipments, UPS domestic and UPS international shipments.
    """
    street_no: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('street_no'), 'exclude': lambda f: f is None }})
    r"""Street number of the addressed building.
    This field can be included in street1 for all carriers except for DHL Germany.
    """
    city: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('city'), 'exclude': lambda f: f is None }})
    r"""**required for purchase**<br>
    Name of a city. When creating a Quote Address, sending a city is optional but will yield more accurate Rates. 
    Please bear in mind that city names may be ambiguous (there are 34 Springfields in the US). Pass in a state 
    or a ZIP code (see below), if known, it will yield more accurate results.
    """
    state: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('state'), 'exclude': lambda f: f is None }})
    r"""**required for purchase for some countries**<br>
    State/Province values are required for shipments from/to the US, AU, and CA. UPS requires province for some 
    countries (i.e Ireland). To receive more accurate quotes, passing this field is recommended. Most carriers 
    only accept two or three character state abbreviations.
    """
    zip: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('zip'), 'exclude': lambda f: f is None }})
    r"""**required for purchase**<br>
    Postal code of an Address. When creating a Quote Addresses, sending a ZIP is optional but will yield more 
    accurate Rates.
    """
    phone: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('phone'), 'exclude': lambda f: f is None }})
    r"""Addresses containing a phone number allow carriers to call the recipient when delivering the Parcel. This
    increases the probability of delivery and helps to avoid accessorial charges after a Parcel has been shipped.
    """
    email: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('email'), 'exclude': lambda f: f is None }})
    r"""E-mail address of the contact person, RFC3696/5321-compliant."""
    is_residential: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('is_residential'), 'exclude': lambda f: f is None }})
    metadata: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    r"""A string of up to 100 characters that can be filled with any additional information you want
    to attach to the object.
    """
    is_complete: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('is_complete'), 'exclude': lambda f: f is None }})
    r"""Complete addresses contain all required values.<br><br>Incomplete addresses have failed one or multiple
    validations.<br>Incomplete Addresses are eligible for requesting rates but lack at least one required 
    value for purchasing labels.
    """
    latitude: Optional[Latitude] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('latitude'), 'exclude': lambda f: f is None }})
    r"""Latitude of address"""
    longitude: Optional[Longitude] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('longitude'), 'exclude': lambda f: f is None }})
    r"""Longitude of address"""
    object_created: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_created'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Date and time of Address creation."""
    object_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_id'), 'exclude': lambda f: f is None }})
    r"""Unique identifier of the given Address object.
    This ID is required to create a Shipment object.
    """
    object_owner: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_owner'), 'exclude': lambda f: f is None }})
    r"""Username of the user who created the Address object."""
    object_updated: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_updated'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Date and time of last Address update. Since you cannot update Addresses after they were created, this time
    stamp reflects the time when the Address was changed by Shippo's systems for the last time, e.g., during the 
    approximation of one or more values.
    """
    validation_results: Optional[AddressValidationResults] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('validation_results'), 'exclude': lambda f: f is None }})
    r"""Object that contains information regarding if an address had been validated or not. Also contains any messages
    generated during validation. Children keys are <code>is_valid</code>(boolean) and <code>messages</code>(array).
    """
    test: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('test'), 'exclude': lambda f: f is None }})
    r"""Indicates whether the object has been created in test mode."""
    


Latitude = Union[float, str]

Longitude = Union[float, str]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelUPSEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | ups_standard | Standard℠|
    | ups_ground | Ground|
    | ups_saver | Saver®|
    | ups_3_day_select | 3 Day Select®|
    | ups_second_day_air | 2nd Day Air®|
    | ups_second_day_air_am | 2nd Day Air® A.M.|
    | ups_next_day_air | Next Day Air®|
    | ups_next_day_air_saver | Next Day Air Saver®|
    | ups_next_day_air_early_am | Next Day Air® Early|
    | ups_mail_innovations_bpm_flat | Mail Innovations BPM Flat |
    | ups_mail_innovations_bpm_parcel | Mail Innovations BPM Parcel |
    | ups_mail_innovations_irregular | Mail Innovations Irregular |
    | ups_mail_innovations_media_mail | Mail Innovations Media Mail |
    | ups_mail_innovations_parcel | Mail Innovations Parcel |
    | ups_mail_innovations_standard_flat | Mail Innovations Standard Flat |
    | ups_surepost | Surepost|
    | ups_surepost_bound_printed_matter | SurePost® Bound Printed Matter|
    | ups_surepost_lightweight | Surepost Lightweight|
    | ups_surepost_media | SurePost® Media|
    | ups_express | Express®|
    | ups_express_1200 | Express 12:00|
    | ups_express_plus | Express Plus®|
    | ups_expedited | Expedited®|
    | ups_express_early | Express® Early|
    | ups_access_point_economy | Access Point™ Economy|
    """
    UPS_STANDARD = 'ups_standard'
    UPS_GROUND = 'ups_ground'
    UPS_SAVER = 'ups_saver'
    UPS_3_DAY_SELECT = 'ups_3_day_select'
    UPS_SECOND_DAY_AIR = 'ups_second_day_air'
    UPS_SECOND_DAY_AIR_AM = 'ups_second_day_air_am'
    UPS_NEXT_DAY_AIR = 'ups_next_day_air'
    UPS_NEXT_DAY_AIR_SAVER = 'ups_next_day_air_saver'
    UPS_NEXT_DAY_AIR_EARLY_AM = 'ups_next_day_air_early_am'
    UPS_MAIL_INNOVATIONS_BPM_FLAT = 'ups_mail_innovations_bpm_flat'
    UPS_MAIL_INNOVATIONS_BPM_PARCEL = 'ups_mail_innovations_bpm_parcel'
    UPS_MAIL_INNOVATIONS_IRREGULAR = 'ups_mail_innovations_irregular'
    UPS_MAIL_INNOVATIONS_MEDIA_MAIL = 'ups_mail_innovations_media_mail'
    UPS_MAIL_INNOVATIONS_PARCEL = 'ups_mail_innovations_parcel'
    UPS_MAIL_INNOVATIONS_STANDARD_FLAT = 'ups_mail_innovations_standard_flat'
    UPS_SUREPOST = 'ups_surepost'
    UPS_SUREPOST_BOUND_PRINTED_MATTER = 'ups_surepost_bound_printed_matter'
    UPS_SUREPOST_LIGHTWEIGHT = 'ups_surepost_lightweight'
    UPS_SUREPOST_MEDIA = 'ups_surepost_media'
    UPS_EXPRESS = 'ups_express'
    UPS_EXPRESS_1200 = 'ups_express_1200'
    UPS_EXPRESS_PLUS = 'ups_express_plus'
    UPS_EXPEDITED = 'ups_expedited'
    UPS_EXPRESS_EARLY = 'ups_express_early'
    UPS_ACCESS_POINT_ECONOMY = 'ups_access_point_economy'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelDPDDEEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | dpd_de_classic | DPD Classic|
    """
    DPD_DE_CLASSIC = 'dpd_de_classic'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from typing import Optional


@dataclasses.dataclass
class GetAddressGlobals:
    shippo_api_version: Optional[str] = dataclasses.field(default=None, metadata={'header': { 'field_name': 'SHIPPO-API-VERSION', 'style': 'simple', 'explode': False }})
    r"""Optional string used to pick a non-default API version to use. See our <a href=\\"https://docs.goshippo.com/docs/api_concepts/apiversioning/\\">API version</a> guide."""
    



@dataclasses.dataclass
class GetAddressRequest:
    address_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'AddressId', 'style': 'simple', 'explode': False }})
    r"""Object ID of the address"""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelGlobegisticsEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | globegistics_priority_mail_express_international | Globegistics Priority Mail Express International|
    | globegistics_priority_mail_international | Globegistics Priority Mail International|
    | globegistics_priority_mail_express_international_pds | Globegistics Priority Mail Express International PreSort Drop Ship|
    | globegistics_priority_mail_international_pds | Globegistics Priority Mail International PreSort Drop Ship|
    | globegistics_epacket | Globegistics ePacket|
    | globegistics_ecom_tracked_ddp | Globegistics eCom Tracked DDP|
    | globegistics_ecom_packet_ddp | Globegistics eCom Packet DDP|
    | globegistics_ecom_priority_mail_international_ddp | Globegistics eCom Priority Mail International DDP|
    | globegistics_ecom_priority_mail_express_international_ddp | Globegistics eCom Priority Mail Express International DDP|
    | globegistics_ecom_extra | Globegistics eCom Extra|
    | globegistics_ecom_international_priority_airmail | Globegistics eCom International Priority Airmail|
    | globegistics_ecom_international_surface_airlift | Globegistics eCom International Surface Air Lift|
    """
    GLOBEGISTICS_PRIORITY_MAIL_EXPRESS_INTERNATIONAL = 'globegistics_priority_mail_express_international'
    GLOBEGISTICS_PRIORITY_MAIL_INTERNATIONAL = 'globegistics_priority_mail_international'
    GLOBEGISTICS_PRIORITY_MAIL_EXPRESS_INTERNATIONAL_PDS = 'globegistics_priority_mail_express_international_pds'
    GLOBEGISTICS_PRIORITY_MAIL_INTERNATIONAL_PDS = 'globegistics_priority_mail_international_pds'
    GLOBEGISTICS_EPACKET = 'globegistics_epacket'
    GLOBEGISTICS_ECOM_TRACKED_DDP = 'globegistics_ecom_tracked_ddp'
    GLOBEGISTICS_ECOM_PACKET_DDP = 'globegistics_ecom_packet_ddp'
    GLOBEGISTICS_ECOM_PRIORITY_MAIL_INTERNATIONAL_DDP = 'globegistics_ecom_priority_mail_international_ddp'
    GLOBEGISTICS_ECOM_PRIORITY_MAIL_EXPRESS_INTERNATIONAL_DDP = 'globegistics_ecom_priority_mail_express_international_ddp'
    GLOBEGISTICS_ECOM_EXTRA = 'globegistics_ecom_extra'
    GLOBEGISTICS_ECOM_INTERNATIONAL_PRIORITY_AIRMAIL = 'globegistics_ecom_international_priority_airmail'
    GLOBEGISTICS_ECOM_INTERNATIONAL_SURFACE_AIRLIFT = 'globegistics_ecom_international_surface_airlift'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from .objectstateenum import ObjectStateEnum
from .weightunitenum import WeightUnitEnum
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CustomsItem:
    description: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('description') }})
    r"""Text description of your item."""
    mass_unit: WeightUnitEnum = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('mass_unit') }})
    r"""The unit used for weight."""
    net_weight: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('net_weight') }})
    r"""Total weight of this item, i.e. quantity * weight per item."""
    origin_country: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('origin_country') }})
    r"""Country of origin of the item. Example: `US` or `DE`.
    All accepted values can be found on the <a href=\"http://www.iso.org/\" target=\"_blank\">Official ISO Website</a>.
    """
    quantity: int = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('quantity') }})
    r"""Quantity of this item in the shipment you send.  Must be greater than 0."""
    value_amount: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('value_amount') }})
    r"""Total value of this item, i.e. quantity * value per item."""
    value_currency: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('value_currency') }})
    r"""Currency used for value_amount. The <a href=\\"http://www.xe.com/iso4217.php\\">official ISO 4217</a>
    currency codes are used, e.g.  `USD` or `EUR`.
    """
    eccn_ear99: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('eccn_ear99'), 'exclude': lambda f: f is None }})
    r"""Export Control Classification Number, required on some exports from the United States."""
    metadata: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    r"""A string of up to 100 characters that can be filled with any additional information you
    want to attach to the object.
    """
    sku_code: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('sku_code'), 'exclude': lambda f: f is None }})
    r"""SKU code of the item, which is required by some carriers."""
    hs_code: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('hs_code'), 'exclude': lambda f: f is None }})
    r"""HS code of the item, which is required by some carriers. If `tariff_number` is not provided, `hs_code` will be used.  If both `hs_code` and `tariff_number` are provided, `tariff_number` will be used. 50 character limit."""
    tariff_number: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('tariff_number'), 'exclude': lambda f: f is None }})
    r"""The tariff number of the item. If `tariff_number` is not provided, `hs_code` will be used. If both `hs_code` and `tariff_number` are provided, `tariff_number` will be used. 12 character limit."""
    object_created: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_created'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Date and time of object creation."""
    object_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_id'), 'exclude': lambda f: f is None }})
    r"""Unique identifier of the given object."""
    object_owner: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_owner'), 'exclude': lambda f: f is None }})
    r"""Username of the user who created the object."""
    object_state: Optional[ObjectStateEnum] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_state'), 'exclude': lambda f: f is None }})
    r"""Indicates the validity of the enclosing object"""
    object_updated: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_updated'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Date and time of last object update."""
    test: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('test'), 'exclude': lambda f: f is None }})
    r"""Indicates whether the object has been created in test mode."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelAsendiaEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | asendia_us_priority_tracked | Asendia USA Priority Tracked|
    | asendia_us_international_express | Asendia USA International Express|
    | asendia_us_international_priority_airmail | Asendia USA International Priority Airmail|
    | asendia_us_international_surface_airlift | Asendia USA International Surface Air Lift|
    | asendia_us_priority_mail_international | Asendia USA Priority Mail International|
    | asendia_us_priority_mail_express_international | Asendia USA Priority Mail Express International|
    | asendia_us_epacket | Asendia USA International ePacket|
    | asendia_us_other | Asendia USA Other Services (custom)|
    """
    ASENDIA_US_PRIORITY_TRACKED = 'asendia_us_priority_tracked'
    ASENDIA_US_INTERNATIONAL_EXPRESS = 'asendia_us_international_express'
    ASENDIA_US_INTERNATIONAL_PRIORITY_AIRMAIL = 'asendia_us_international_priority_airmail'
    ASENDIA_US_INTERNATIONAL_SURFACE_AIRLIFT = 'asendia_us_international_surface_airlift'
    ASENDIA_US_PRIORITY_MAIL_INTERNATIONAL = 'asendia_us_priority_mail_international'
    ASENDIA_US_PRIORITY_MAIL_EXPRESS_INTERNATIONAL = 'asendia_us_priority_mail_express_international'
    ASENDIA_US_EPACKET = 'asendia_us_epacket'
    ASENDIA_US_OTHER = 'asendia_us_other'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelVehoEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | veho_next_day | Veho Next Day |
    """
    VEHO_NEXT_DAY = 'veho_next_day'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .webhookpayloadbatch import WebhookPayloadBatch
from .webhookpayloadtrack import WebhookPayloadTrack
from .webhookpayloadtransaction import WebhookPayloadTransaction
from typing import Union

WebhookPayload = Union[WebhookPayloadTrack, WebhookPayloadBatch, WebhookPayloadTransaction]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CustomsDeclarationNonDeliveryOptionEnum(str, Enum):
    r"""Indicates how the carrier should proceed in case the shipment can't be delivered.
    Allowed values available <a href=\"#tag/Customs-Declaration-Non-Delivery-Option\">here</a>
    """
    ABANDON = 'ABANDON'
    RETURN = 'RETURN'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelFedExEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | fedex_ground | FedEx Ground®|
    | fedex_home_delivery | FedEx Home Delivery®|
    | fedex_ground_economy | FedEx Ground® Economy|
    | fedex_2_day | FedEx 2Day®|
    | fedex_2_day_am | FedEx 2Day® A.M.|
    | fedex_express_saver | FedEx Express Saver®|
    | fedex_standard_overnight | FedEx Standard Overnight®|
    | fedex_priority_overnight | FedEx Priority Overnight®|
    | fedex_first_overnight | FedEx First Overnight®|
    | fedex_international_economy | FedEx International Economy®|
    | fedex_international_priority | FedEx International Priority®|
    | fedex_international_first | FedEx International First®|
    | fedex_europe_first_international_priority | FedEx International First®|
    | fedex_international_connect_plus | FedEx International Connect Plus|
    | fedex_first | FedEx First |
    | fedex_priority | FedEx Priority |
    | fedex_priority_express | FedEx Priority Express|
    | fedex_economy_select | FedEx® Economy |
    | fedex_regional_economy | FedEx® Regional Economy |
    | fedex_first_overnight_extra_hours | First Overnight® EH |
    | fedex_international_priority_express | International Priority® Express |
    | fedex_next_day_mid_morning | Next Day Mid Morning |
    | fedex_priority_overnight_extra_hours | Priority Overnight® EH |
    | fedex_standard_overnight_extra_hours | Standard Overnight® EH |
    """
    FEDEX_GROUND = 'fedex_ground'
    FEDEX_HOME_DELIVERY = 'fedex_home_delivery'
    FEDEX_SMART_POST = 'fedex_smart_post'
    FEDEX_GROUND_ECONOMY = 'fedex_ground_economy'
    FEDEX_2_DAY = 'fedex_2_day'
    FEDEX_2_DAY_AM = 'fedex_2_day_am'
    FEDEX_EXPRESS_SAVER = 'fedex_express_saver'
    FEDEX_STANDARD_OVERNIGHT = 'fedex_standard_overnight'
    FEDEX_PRIORITY_OVERNIGHT = 'fedex_priority_overnight'
    FEDEX_FIRST_OVERNIGHT = 'fedex_first_overnight'
    FEDEX_INTERNATIONAL_ECONOMY = 'fedex_international_economy'
    FEDEX_INTERNATIONAL_PRIORITY = 'fedex_international_priority'
    FEDEX_INTERNATIONAL_FIRST = 'fedex_international_first'
    FEDEX_EUROPE_FIRST_INTERNATIONAL_PRIORITY = 'fedex_europe_first_international_priority'
    FEDEX_INTERNATIONAL_CONNECT_PLUS = 'fedex_international_connect_plus'
    FEDEX_FIRST = 'fedex_first'
    FEDEX_PRIORITY = 'fedex_priority'
    FEDEX_PRIORITY_EXPRESS = 'fedex_priority_express'
    FEDEX_ECONOMY_SELECT = 'fedex_economy_select'
    FEDEX_REGIONAL_ECONOMY = 'fedex_regional_economy'
    FEDEX_FIRST_OVERNIGHT_EXTRA_HOURS = 'fedex_first_overnight_extra_hours'
    FEDEX_INTERNATIONAL_PRIORITY_EXPRESS = 'fedex_international_priority_express'
    FEDEX_NEXT_DAY_MID_MORNING = 'fedex_next_day_mid_morning'
    FEDEX_PRIORITY_OVERNIGHT_EXTRA_HOURS = 'fedex_priority_overnight_extra_hours'
    FEDEX_STANDARD_OVERNIGHT_EXTRA_HOURS = 'fedex_standard_overnight_extra_hours'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from typing import Dict, List, Optional


@dataclasses.dataclass
class InitiateOauth2SigninGlobals:
    shippo_api_version: Optional[str] = dataclasses.field(default=None, metadata={'header': { 'field_name': 'SHIPPO-API-VERSION', 'style': 'simple', 'explode': False }})
    r"""Optional string used to pick a non-default API version to use. See our <a href=\\"https://docs.goshippo.com/docs/api_concepts/apiversioning/\\">API version</a> guide."""
    



@dataclasses.dataclass
class InitiateOauth2SigninRequest:
    carrier_account_object_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'CarrierAccountObjectId', 'style': 'simple', 'explode': False }})
    r"""The carrier account ID (UUID) to start a signin process."""
    redirect_uri: str = dataclasses.field(metadata={'query_param': { 'field_name': 'redirect_uri', 'style': 'form', 'explode': True }})
    r"""Callback URL. The URL that tells the authorization server where to send the user back to after they approve the request."""
    state: Optional[str] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'state', 'style': 'form', 'explode': True }})
    r"""A random string generated by the consuming application and included in the request to prevent CSRF attacks. The consuming application checks that the same value is returned after the user authorizes Shippo."""
    



@dataclasses.dataclass
class InitiateOauth2SigninResponse:
    headers: Dict[str, List[str]] = dataclasses.field()
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .parceltemplatearamexaustraliaenum import ParcelTemplateAramexAustraliaEnum
from .parceltemplatecourierspleaseenum import ParcelTemplateCouriersPleaseEnum
from .parceltemplatedhlecommerceenum import ParcelTemplateDHLeCommerceEnum
from .parceltemplatedpdukenum import ParcelTemplateDPDUKEnum
from .parceltemplatefedexenum import ParcelTemplateFedExEnum
from .parceltemplateupsenum import ParcelTemplateUPSEnum
from .parceltemplateuspsenum import ParcelTemplateUSPSEnum
from typing import Union

ParcelTemplateEnumSet = Union[ParcelTemplateFedExEnum, ParcelTemplateUPSEnum, ParcelTemplateUSPSEnum, ParcelTemplateDHLeCommerceEnum, ParcelTemplateDPDUKEnum, ParcelTemplateCouriersPleaseEnum, ParcelTemplateAramexAustraliaEnum]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import requests as requests_http
from .sdkconfiguration import SDKConfiguration
from shippo import utils
from shippo._hooks import AfterErrorContext, AfterSuccessContext, BeforeRequestContext, HookContext
from shippo.models import components, errors, operations
from typing import Optional

class Rates:
    r"""A rate is the cost to ship a parcel from a carrier. The rate object details the service level including the cost and transit time.
    <SchemaDefinition schemaRef=\"#/components/schemas/Rate\"/>
    """
    sdk_configuration: SDKConfiguration

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        self.sdk_configuration = sdk_config
        
    
    
    def get(self, rate_id: str) -> components.Rate:
        r"""Retrieve a rate
        Returns an existing rate using a rate object ID.
        """
        hook_ctx = HookContext(operation_id='GetRate', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.GetRateRequest(
            rate_id=rate_id,
        )
        
        _globals = operations.GetRateGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(base_url, '/rates/{RateId}', request, _globals)
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('GET', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.Rate])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def list_shipment_rates(self, shipment_id: str, page: Optional[int] = None, results: Optional[int] = None) -> components.RatePaginatedList:
        r"""Retrieve shipment rates
        Returns a paginated list of rates associated with a shipment
        """
        hook_ctx = HookContext(operation_id='ListShipmentRates', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.ListShipmentRatesRequest(
            shipment_id=shipment_id,
            page=page,
            results=results,
        )
        
        _globals = operations.ListShipmentRatesGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(base_url, '/shipments/{ShipmentId}/rates', request, _globals)
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        query_params = { **utils.get_query_params(request, _globals), **query_params }
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('GET', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.RatePaginatedList])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def list_shipment_rates_by_currency_code(self, request: operations.ListShipmentRatesByCurrencyCodeRequest) -> components.RatePaginatedList:
        r"""Retrieve shipment rates in currency
        Returns all available shipping rates for a shipment object.

        When you create a new valid shipment object, Shippo automatically calculates all available rates. Depending on your shipment data, there may be none, one or multiple rates.

        By default, the calculated rates will return the price in two currencies under the `amount` and `amount_local` keys, respectively. The `amount` key will contain the price of a rate expressed in the currency that is used in the country from where the parcel originates, and the `amount_local` key will contain the price expressed in the currency that is used in the country the parcel is shipped to. You can request rates with prices expressed in a different currency by adding the currency code to the end of the resource URL. The full list of supported currencies along with their codes can be viewed on <a href=\"http://openexchangerates.org/api/currencies.json\">open exchange rates</a>.

        Note: re-requesting the rates with a different currency code will re-queue the shipment (i.e. set the Shipment's `status` to `QUEUED`) and the converted currency rates will only be available when the Shipment's `status` is set to `SUCCESS`.
        """
        hook_ctx = HookContext(operation_id='ListShipmentRatesByCurrencyCode', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        _globals = operations.ListShipmentRatesByCurrencyCodeGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(base_url, '/shipments/{ShipmentId}/rates/{CurrencyCode}', request, _globals)
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        query_params = { **utils.get_query_params(request, _globals), **query_params }
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('GET', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.RatePaginatedList])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from ...models.components import ordershopappenum as components_ordershopappenum
from ...models.components import orderstatusenum as components_orderstatusenum
from typing import List, Optional


@dataclasses.dataclass
class ListOrdersGlobals:
    shippo_api_version: Optional[str] = dataclasses.field(default=None, metadata={'header': { 'field_name': 'SHIPPO-API-VERSION', 'style': 'simple', 'explode': False }})
    r"""Optional string used to pick a non-default API version to use. See our <a href=\\"https://docs.goshippo.com/docs/api_concepts/apiversioning/\\">API version</a> guide."""
    



@dataclasses.dataclass
class ListOrdersRequest:
    page: Optional[int] = dataclasses.field(default=1, metadata={'query_param': { 'field_name': 'page', 'style': 'form', 'explode': True }})
    r"""The page number you want to select"""
    results: Optional[int] = dataclasses.field(default=25, metadata={'query_param': { 'field_name': 'results', 'style': 'form', 'explode': True }})
    r"""The number of results to return per page (max 100)"""
    order_status: Optional[List[components_orderstatusenum.OrderStatusEnum]] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'order_status[]', 'style': 'form', 'explode': True }})
    r"""Filter orders by order status"""
    shop_app: Optional[components_ordershopappenum.OrderShopAppEnum] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'shop_app', 'style': 'form', 'explode': True }})
    r"""Filter orders by shop app"""
    start_date: Optional[str] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'start_date', 'style': 'form', 'explode': True }})
    r"""Filter orders created after the input date and time (ISO 8601 UTC format).  This is based on the
    `placed_at` field, meaning when the order has been placed, not when the order object was created.
    """
    end_date: Optional[str] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'end_date', 'style': 'form', 'explode': True }})
    r"""Filter orders created before the input date and time (ISO 8601 UTC format).  This is based on the
    `placed_at` field, meaning when the order has been placed, not when the order object was created.
    """
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CustomsDeclarationEelPfcEnum(str, Enum):
    r"""EEL / PFC type of the shipment. For most shipments from the US to CA, `NOEEI_30_36` is applicable; for most
    other shipments from the US, `NOEEI_30_37_a` is applicable.
    Allowed values available <a href=\"#tag/Customs-Declaration-EELPFC\">here</a>
    """
    NOEEI_30_37_A = 'NOEEI_30_37_a'
    NOEEI_30_37_H = 'NOEEI_30_37_h'
    NOEEI_30_37_F = 'NOEEI_30_37_f'
    NOEEI_30_36 = 'NOEEI_30_36'
    AES_ITN = 'AES_ITN'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .userparceltemplatewithcarriertemplatecreaterequest import UserParcelTemplateWithCarrierTemplateCreateRequest
from .userparceltemplatewithoutcarriertemplatecreaterequest import UserParcelTemplateWithoutCarrierTemplateCreateRequest
from typing import Union

UserParcelTemplateCreateRequest = Union[UserParcelTemplateWithCarrierTemplateCreateRequest, UserParcelTemplateWithoutCarrierTemplateCreateRequest]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ParcelTemplateAramexAustraliaEnum(str, Enum):
    r"""|Token | Name | Dimensions|
    |:---|:---|:---|
    | Fastway_Australia_Satchel_A2 | Satchel A2 | 594.00 x 420.00 x 48.00 mm|
    | Fastway_Australia_Satchel_A3 | Satchel A3 | 420.00 x 297.00 x 64.00 mm|
    | Fastway_Australia_Satchel_A4 | Satchel A4 | 297.00 x 210.00 x 64.00 mm|
    | Fastway_Australia_Satchel_A5 | Satchel A5 | 210.00 x 148.00 x 64.00 mm|
    """
    FASTWAY_AUSTRALIA_SATCHEL_A2 = 'Fastway_Australia_Satchel_A2'
    FASTWAY_AUSTRALIA_SATCHEL_A3 = 'Fastway_Australia_Satchel_A3'
    FASTWAY_AUSTRALIA_SATCHEL_A4 = 'Fastway_Australia_Satchel_A4'
    FASTWAY_AUSTRALIA_SATCHEL_A5 = 'Fastway_Australia_Satchel_A5'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WebhookEventTypeEnum(str, Enum):
    r"""Type of event that triggered the webhook."""
    TRANSACTION_CREATED = 'transaction_created'
    TRANSACTION_UPDATED = 'transaction_updated'
    TRACK_UPDATED = 'track_updated'
    BATCH_CREATED = 'batch_created'
    BATCH_PURCHASED = 'batch_purchased'
    ALL = 'all'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class OrderShopAppEnum(str, Enum):
    r"""Platform the order was created on and, if applicable, imported from.
    Orders created via the Shippo API or dashboard will have the value \"Shippo\".
    """
    AMAZON = 'Amazon'
    BIGCOMMERCE = 'Bigcommerce'
    CSV_IMPORT = 'CSV_Import'
    E_BAY = 'eBay'
    E_PAGES = 'ePages'
    ETSY = 'Etsy'
    GODADDY = 'Godaddy'
    MAGENTO = 'Magento'
    SHIPPO = 'Shippo'
    SHOPIFY = 'Shopify'
    SPREECOMMERCE = 'Spreecommerce'
    STRIPE_RELAY = 'StripeRelay'
    WALMART = 'Walmart'
    WEEBLY = 'Weebly'
    WOO_COMMERCE = 'WooCommerce'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from shippo import utils
from typing import Optional


class ParcelInsuranceProvider(str, Enum):
    r"""To have insurance cover provided by a carrier directly instead of Shippo's provider (XCover), set provider to `FEDEX`, `UPS`, or `ONTRAC`."""
    FEDEX = 'FEDEX'
    UPS = 'UPS'
    ONTRAC = 'ONTRAC'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class ParcelInsurance:
    r"""To add insurace to your parcel, specify `amount`, `content` and `currency`. <br><br>If you do not want to add insurance to you parcel, do not set these parameters."""
    amount: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('amount'), 'exclude': lambda f: f is None }})
    r"""Declared value of the goods you want to insure."""
    content: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('content'), 'exclude': lambda f: f is None }})
    r"""Description of parcel content."""
    currency: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('currency'), 'exclude': lambda f: f is None }})
    r"""Currency for the amount value. Currently only USD is supported for FedEx and UPS."""
    provider: Optional[ParcelInsuranceProvider] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('provider'), 'exclude': lambda f: f is None }})
    r"""To have insurance cover provided by a carrier directly instead of Shippo's provider (XCover), set provider to `FEDEX`, `UPS`, or `ONTRAC`."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Webhook:
    event: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event') }})
    r"""Type of event that triggers the webhook."""
    url: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('url') }})
    r"""URL webhook events are sent to."""
    active: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('active'), 'exclude': lambda f: f is None }})
    r"""Determines whether the webhook is active or not."""
    is_test: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('is_test'), 'exclude': lambda f: f is None }})
    r"""Determines whether the webhook is a test webhook or not."""
    object_created: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_created'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Timestamp of the creation of the webhook."""
    object_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_id'), 'exclude': lambda f: f is None }})
    r"""Unique identifier of the webhook. This can be used to retrieve or delete the webhook."""
    object_updated: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_updated'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Timestamp of the last update of the webhook."""
    object_owner: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_owner'), 'exclude': lambda f: f is None }})
    r"""Username of the user who created the webhook."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class AddressValidationResultsMessageCodeEnum(str, Enum):
    VERIFICATION_ERROR = 'verification_error'
    UNKNOWN_STREET_LOWER = 'unknown_street'
    COMPONENT_MISMATCH_ERROR = 'component_mismatch_error'
    MULTIPLE_MATCH = 'multiple_match'
    SUB_PREMISE_NUMBER_INVALID = 'sub_premise_number_invalid'
    SUB_PREMISE_NUMBER_MISSING = 'sub_premise_number_missing'
    PREMISE_NUMBER_INVALID = 'premise_number_invalid'
    PREMISE_NUMBER_MISSING = 'premise_number_missing'
    BOX_NUMBER_INVALID = 'box_number_invalid'
    BOX_NUMBER_MISSING = 'box_number_missing'
    PMB_NUMBER_MISSING = 'pmb_number_missing'
    POSTAL_CODE_CHANGE = 'postal_code_change'
    ADMINISTRATIVE_AREA_CHANGE = 'administrative_area_change'
    LOCALITY_CHANGE = 'locality_change'
    DEPENDENT_LOCALITY_CHANGE = 'dependent_locality_change'
    STREET_NAME_CHANGE = 'street_name_change'
    STREET_TYPE_CHANGE = 'street_type_change'
    STREET_DIRECTIONAL_CHANGE = 'street_directional_change'
    SUB_PREMISE_TYPE_CHANGE = 'sub_premise_type_change'
    SUB_PREMISE_NUMBER_CHANGE = 'sub_premise_number_change'
    DOUBLE_DEPENDENT_LOCALITY_CHANGE = 'double_dependent_locality_change'
    SUBADMINISTRATIVE_AREA_CHANGE = 'subadministrative_area_change'
    SUBNATIONAL_AREA_CHANGE = 'subnational_area_change'
    PO_BOX_CHANGE = 'po_box_change'
    PREMISE_TYPE_CHANGE = 'premise_type_change'
    HOUSE_NUMBER_CHANGE = 'house_number_change'
    ORGANIZATION_CHANGE = 'organization_change'
    EXTRANEOUS_INFORMATION = 'extraneous_information'
    USPS_DOOR_INACCESSIBLE = 'usps_door_inaccessible'
    ADMINISTRATIVE_AREA_PARTIAL = 'administrative_area_partial'
    CITY_PARTIAL = 'city_partial'
    STREET_PARTIAL = 'street_partial'
    BUILDING_PARTIAL = 'building_partial'
    SUBPREMISE_PARTIAL = 'subpremise_partial'
    ADMINISTRATIVE_AREA_FULL = 'administrative_area_full'
    CITY_FULL = 'city_full'
    THOROUGHFARE_FULL = 'thoroughfare_full'
    PREMISES_FULL = 'premises_full'
    SUBPREMISE_FULL = 'subpremise_full'
    GEOCODED_STREET = 'geocoded_street'
    GEOCODED_NEIGHBORHOOD = 'geocoded_neighborhood'
    GEOCODED_COMMUNITY = 'geocoded_community'
    GEOCODED_STATE = 'geocoded_state'
    GEOCODED_ROOFTOP = 'geocoded_rooftop'
    GEOCODED_INTERPOLATED_ROOFTOP = 'geocoded_interpolated_rooftop'
    INVALID_POSTAL_CODE = 'invalid_postal_code'
    POSTAL_CODE_NOT_FOUND = 'postal_code_not_found'
    EMPTY_REQUEST = 'empty_request'
    SERVICE_ERROR = 'service_error'
    STREET_DETAIL_MISSING = 'street_detail_missing'
    INVALID_CITY_STATE_ZIP = 'Invalid City/State/Zip'
    DEFAULT_MATCH = 'Default Match'
    UNKNOWN_STREET_MIXED = 'Unknown Street'
    ADDRESS_NOT_FOUND = 'Address Not Found'
    NON_DELIVERABLE_ZIP4 = 'Non-Deliverable ZIP4'
    MULTIPLE_RESPONSES = 'Multiple Responses'
    INVALID_DUAL_ADDRESS = 'Invalid Dual Address'
    INVALID_STATE = 'Invalid State'
    INVALID_CITY = 'Invalid City'
    AMBIGUOUS_ADDRESS = 'Ambiguous Address'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from shippo import utils
from typing import Optional


class RecipientType(str, Enum):
    r"""Mandatory for Fedex only. License type of the recipient of the Alcohol Package."""
    LICENSEE = 'licensee'
    CONSUMER = 'consumer'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Alcohol:
    r"""Indicates that a shipment contains Alcohol (Fedex and UPS only)."""
    contains_alcohol: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('contains_alcohol'), 'exclude': lambda f: f is None }})
    r"""Mandatory for Fedex and UPS. Specifies that the package contains Alcohol."""
    recipient_type: Optional[RecipientType] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('recipient_type'), 'exclude': lambda f: f is None }})
    r"""Mandatory for Fedex only. License type of the recipient of the Alcohol Package."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from shippo import utils
from typing import Optional


class PaymentMethod(str, Enum):
    r"""Secured funds include money orders, certified cheques and others (see
    <a href=\"https://www.ups.com/content/us/en/shipping/time/service/value_added/cod.html\">UPS</a> for details). 
    If no payment_method inputted the value defaults to \"ANY\".)
    """
    SECURED_FUNDS = 'SECURED_FUNDS'
    CASH = 'CASH'
    ANY = 'ANY'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Cod:
    r"""Specify collection on delivery details (UPS only)."""
    amount: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('amount'), 'exclude': lambda f: f is None }})
    r"""Amount to be collected."""
    currency: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('currency'), 'exclude': lambda f: f is None }})
    r"""Currency for the amount to be collected. Currently only USD is supported for UPS."""
    payment_method: Optional[PaymentMethod] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('payment_method'), 'exclude': lambda f: f is None }})
    r"""Secured funds include money orders, certified cheques and others (see
    <a href=\"https://www.ups.com/content/us/en/shipping/time/service/value_added/cod.html\">UPS</a> for details). 
    If no payment_method inputted the value defaults to \"ANY\".)
    """
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from shippo import utils
from typing import Optional


class Type(str, Enum):
    r"""Party to be billed. (Leave blank for DHL Germany.)"""
    SENDER = 'SENDER'
    RECIPIENT = 'RECIPIENT'
    THIRD_PARTY = 'THIRD_PARTY'
    THIRD_PARTY_CONSIGNEE = 'THIRD_PARTY_CONSIGNEE'
    COLLECT = 'COLLECT'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Billing:
    r"""Specify billing details (UPS, FedEx, and DHL Germany only)."""
    account: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('account'), 'exclude': lambda f: f is None }})
    r"""Account number to be billed. (For DHL Germany, leave this field blank.)"""
    country: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('country'), 'exclude': lambda f: f is None }})
    r"""Country iso2 code of account number to be billed (required for UPS third party billing only)."""
    participation_code: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('participation_code'), 'exclude': lambda f: f is None }})
    r"""2 digit code used to override your default participation code associated with your DHL Germany account."""
    type: Optional[Type] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type'), 'exclude': lambda f: f is None }})
    r"""Party to be billed. (Leave blank for DHL Germany.)"""
    zip: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('zip'), 'exclude': lambda f: f is None }})
    r"""ZIP code of account number to be billed (required for UPS if there is a zip on the billing account)."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelLasershipEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | lasership_routed_delivery | Routed Delivery|
    """
    LASERSHIP_ROUTED_DELIVERY = 'lasership_routed_delivery'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from typing import Optional


@dataclasses.dataclass
class ListShipmentRatesGlobals:
    shippo_api_version: Optional[str] = dataclasses.field(default=None, metadata={'header': { 'field_name': 'SHIPPO-API-VERSION', 'style': 'simple', 'explode': False }})
    r"""Optional string used to pick a non-default API version to use. See our <a href=\\"https://docs.goshippo.com/docs/api_concepts/apiversioning/\\">API version</a> guide."""
    



@dataclasses.dataclass
class ListShipmentRatesRequest:
    shipment_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'ShipmentId', 'style': 'simple', 'explode': False }})
    r"""Object ID of the shipment to update"""
    page: Optional[int] = dataclasses.field(default=1, metadata={'query_param': { 'field_name': 'page', 'style': 'form', 'explode': True }})
    r"""The page number you want to select"""
    results: Optional[int] = dataclasses.field(default=25, metadata={'query_param': { 'field_name': 'results', 'style': 'form', 'explode': True }})
    r"""The number of results to return per page (max 100)"""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .address import *
from .addresscompletecreaterequest import *
from .addresscreaterequest import *
from .addressimporter import *
from .addresspaginatedlist import *
from .addressvalidationresults import *
from .addressvalidationresultsmessage import *
from .addressvalidationresultsmessagecodeenum import *
from .addressvalidationresultsmessagesourceenum import *
from .alcohol import *
from .batch import *
from .batchcreaterequest import *
from .batchshipment import *
from .batchshipmentcreaterequest import *
from .batchshipmentpaginatedlist import *
from .billing import *
from .carrieraccount import *
from .carrieraccountbase import *
from .carrieraccountcanadapostcreateparameters import *
from .carrieraccountcanadapostcreaterequest import *
from .carrieraccountchronopostcreaterequest import *
from .carrieraccountcolissimocreaterequest import *
from .carrieraccountcorreoscreaterequest import *
from .carrieraccountdeutschepostcreaterequest import *
from .carrieraccountdhlexpresscreaterequest import *
from .carrieraccountdhlexpresscreaterequestparameters import *
from .carrieraccountdpddecreaterequest import *
from .carrieraccountdpdukcreaterequest import *
from .carrieraccountfedexcreaterequest import *
from .carrieraccounthermesukcreaterequest import *
from .carrieraccountmondialrelaycreaterequest import *
from .carrieraccountpaginatedlist import *
from .carrieraccountposteitalianecreaterequest import *
from .carrieraccountregistrationstatus import *
from .carrieraccountsendlecreaterequest import *
from .carrieraccountservicelevel import *
from .carrieraccountupscreaterequest import *
from .carrieraccountupscreaterequestparameters import *
from .carrieraccountuspscreaterequest import *
from .carrieraccountwithextrainfo import *
from .carrierparceltemplate import *
from .carrierparceltemplatelist import *
from .carriersenum import *
from .cod import *
from .connectexistingownaccountrequest import *
from .corerate import *
from .customerreference import *
from .customsdeclaration import *
from .customsdeclarationb13afilingoptionenum import *
from .customsdeclarationcontentstypeenum import *
from .customsdeclarationcreaterequest import *
from .customsdeclarationeelpfcenum import *
from .customsdeclarationincotermenum import *
from .customsdeclarationnondeliveryoptionenum import *
from .customsdeclarationpaginatedlist import *
from .customsexporteridentification import *
from .customsinvoicedcharges import *
from .customsitem import *
from .customsitemcreaterequest import *
from .customsitempaginatedlist import *
from .customstaxidentification import *
from .dangerousgoodsbiologicalmaterial import *
from .dangerousgoodslithiumbatteries import *
from .dangerousgoodsobject import *
from .defaultparceltemplate import *
from .defaultparceltemplateupdaterequest import *
from .departmentnumber import *
from .distanceunitenum import *
from .dryice import *
from .fedexconnectexistingownaccountparameters import *
from .instanttransactioncreaterequest import *
from .insurance import *
from .invoicenumber import *
from .labelfiletypeenum import *
from .lineitem import *
from .lineitembase import *
from .liverate import *
from .liveratecreaterequest import *
from .liveratepaginatedlist import *
from .location import *
from .manifest import *
from .manifestcreaterequest import *
from .manifestpaginatedlist import *
from .objectstateenum import *
from .order import *
from .ordercreaterequest import *
from .orderpaginatedlist import *
from .ordershopappenum import *
from .orderstatusenum import *
from .parcel import *
from .parcelcreatefromtemplaterequest import *
from .parcelcreaterequest import *
from .parcelextra import *
from .parcelinsurance import *
from .parcelpaginatedlist import *
from .parceltemplatearamexaustraliaenum import *
from .parceltemplatecourierspleaseenum import *
from .parceltemplatedhlecommerceenum import *
from .parceltemplatedpdukenum import *
from .parceltemplateenumset import *
from .parceltemplatefedexenum import *
from .parceltemplateupsenum import *
from .parceltemplateuspsenum import *
from .pickup import *
from .pickupbase import *
from .ponumber import *
from .rate import *
from .ratepaginatedlist import *
from .refund import *
from .refundpaginatedlist import *
from .refundrequestbody import *
from .responsemessage import *
from .rmanumber import *
from .security import *
from .servicegroup import *
from .servicegroupaccountandservicelevel import *
from .servicegroupcreaterequest import *
from .servicegrouptypeenum import *
from .servicegroupupdaterequest import *
from .servicelevel import *
from .servicelevelairterraenum import *
from .servicelevelapcpostalenum import *
from .servicelevelapgenum import *
from .servicelevelaramexaustraliaenum import *
from .servicelevelasendiaenum import *
from .servicelevelaustraliapostenum import *
from .servicelevelbettertrucksenum import *
from .servicelevelcanadapostenum import *
from .servicelevelcdlenum import *
from .servicelevelchronopostenum import *
from .servicelevelcolissimoenum import *
from .servicelevelcorreosespanaenum import *
from .servicelevelcourierspleaseenum import *
from .serviceleveldeutschepostenum import *
from .serviceleveldhlecommerceenum import *
from .serviceleveldhlexpressenum import *
from .serviceleveldhlgermanyenum import *
from .serviceleveldpddeenum import *
from .serviceleveldpdukenum import *
from .servicelevelenumset import *
from .servicelevelepostglobalenum import *
from .servicelevelevriukenum import *
from .servicelevelfedexenum import *
from .servicelevelglobegisticsenum import *
from .servicelevelglsusenum import *
from .serviceleveljitsuenum import *
from .servicelevellasershipenum import *
from .servicelevellsoenum import *
from .servicelevelmondialrelayenum import *
from .servicelevelontracenum import *
from .servicelevelparcelforceenum import *
from .servicelevelpostitalianeenum import *
from .servicelevelpurolatorenum import *
from .servicelevelroyalmailenum import *
from .servicelevelsendleenum import *
from .servicelevelswyftenum import *
from .serviceleveludsenum import *
from .servicelevelupsenum import *
from .serviceleveluspsenum import *
from .servicelevelvehoenum import *
from .servicelevelwithparent import *
from .shipment import *
from .shipmentcreaterequest import *
from .shipmentextra import *
from .shipmentextralasershipattributesenum import *
from .shipmentextrareturnservicetypelasershipenum import *
from .shipmentextrareturnservicetypeupsenum import *
from .shipmentpaginatedlist import *
from .shippoaccount import *
from .shippoaccountpaginatedlist import *
from .shippoaccountupdaterequest import *
from .track import *
from .trackingstatus import *
from .trackingstatusenum import *
from .trackingstatuslocationbase import *
from .trackingstatussubstatus import *
from .tracksrequest import *
from .transaction import *
from .transactioncreaterequest import *
from .transactionpaginatedlist import *
from .transactionstatusenum import *
from .upsconnectexistingownaccountparameters import *
from .upsreferencefields import *
from .userparceltemplate import *
from .userparceltemplatecreaterequest import *
from .userparceltemplatelist import *
from .userparceltemplateupdaterequest import *
from .userparceltemplatewithcarriertemplatecreaterequest import *
from .userparceltemplatewithoutcarriertemplatecreaterequest import *
from .webhook import *
from .webhookeventtypeenum import *
from .webhookpaginatedlist import *
from .webhookpayload import *
from .webhookpayloadbatch import *
from .webhookpayloadtrack import *
from .webhookpayloadtransaction import *
from .webhookupdaterequest import *
from .weightunitenum import *

__all__ = ["Address","AddressCompleteCreateRequest","AddressCreateRequest","AddressFrom","AddressImporter","AddressPaginatedList","AddressReturn","AddressTo","AddressValidationResults","AddressValidationResultsMessage","AddressValidationResultsMessageCodeEnum","AddressValidationResultsMessageSourceEnum","Alcohol","AncillaryEndorsement","Attributes","Authentication","Batch","BatchCreateRequest","BatchShipment","BatchShipmentCreateRequest","BatchShipmentPaginatedList","BatchStatus","Billing","BuildingLocationType","BuildingType","CarrierAccount","CarrierAccountBase","CarrierAccountBaseParameters","CarrierAccountCanadaPostCreateParameters","CarrierAccountCanadaPostCreateRequest","CarrierAccountChronopostCreateRequest","CarrierAccountChronopostCreateRequestParameters","CarrierAccountColissimoCreateRequest","CarrierAccountColissimoCreateRequestParameters","CarrierAccountCorreosCreateRequest","CarrierAccountCorreosCreateRequestParameters","CarrierAccountDHLExpressCreateRequest","CarrierAccountDHLExpressCreateRequestParameters","CarrierAccountDPDUKCreateRequest","CarrierAccountDPDUKCreateRequestParameters","CarrierAccountDeutschePostCreateRequest","CarrierAccountDeutschePostCreateRequestParameters","CarrierAccountDpdDeCreateRequest","CarrierAccountDpdDeCreateRequestParameters","CarrierAccountFedExCreateRequest","CarrierAccountFedExCreateRequestParameters","CarrierAccountHermesUKCreateRequest","CarrierAccountHermesUKCreateRequestParameters","CarrierAccountMondialRelayCreateRequest","CarrierAccountMondialRelayCreateRequestParameters","CarrierAccountPaginatedList","CarrierAccountParameters","CarrierAccountPosteItalianeCreateRequest","CarrierAccountPosteItalianeCreateRequestParameters","CarrierAccountRegistrationStatus","CarrierAccountSendleCreateRequest","CarrierAccountSendleCreateRequestParameters","CarrierAccountServiceLevel","CarrierAccountUPSCreateRequest","CarrierAccountUPSCreateRequestParameters","CarrierAccountUSPSCreateRequest","CarrierAccountUSPSCreateRequestParameters","CarrierAccountWithExtraInfo","CarrierAccountWithExtraInfoStatus","CarrierAccountWithExtraInfoType","CarrierParcelTemplate","CarrierParcelTemplateList","CarriersEnum","Cod","ConnectExistingOwnAccountRequest","ConnectExistingOwnAccountRequestParameters","CoreRate","CreatedBy","CustomerReference","CustomsDeclaration","CustomsDeclarationAddress","CustomsDeclarationB13AFilingOptionEnum","CustomsDeclarationContentsTypeEnum","CustomsDeclarationCreateRequest","CustomsDeclarationCreateRequestAddress","CustomsDeclarationCreateRequestType","CustomsDeclarationDutiesPayor","CustomsDeclarationEelPfcEnum","CustomsDeclarationIncotermEnum","CustomsDeclarationNonDeliveryOptionEnum","CustomsDeclarationPaginatedList","CustomsDeclarationType","CustomsExporterIdentification","CustomsInvoicedCharges","CustomsItem","CustomsItemCreateRequest","CustomsItemPaginatedList","CustomsTaxIdentification","CustomsTaxIdentificationType","DangerousGoodsBiologicalMaterial","DangerousGoodsCode","DangerousGoodsLithiumBatteries","DangerousGoodsObject","DefaultParcelTemplate","DefaultParcelTemplateUpdateRequest","DepartmentNumber","DistanceUnitEnum","DryIce","DutiesPayor","FedExConnectExistingOwnAccountParameters","InstantTransactionCreateRequest","Insurance","InvoiceNumber","LabelFileType","LabelFileTypeEnum","Latitude","LineItem","LineItemBase","LiveRate","LiveRateCreateRequest","LiveRateCreateRequestAddressFrom","LiveRateCreateRequestAddressTo","LiveRateCreateRequestParcel","LiveRatePaginatedList","Location","Longitude","Manifest","ManifestCreateRequest","ManifestCreateRequestAddressFrom","ManifestPaginatedList","ManifestStatus","ObjectInfo","ObjectResults","ObjectState","ObjectStateEnum","Order","OrderCreateRequest","OrderPaginatedList","OrderShopAppEnum","OrderStatusEnum","Parameters","Parcel","ParcelCreateFromTemplateRequest","ParcelCreateRequest","ParcelExtra","ParcelInsurance","ParcelInsuranceProvider","ParcelPaginatedList","ParcelTemplateAramexAustraliaEnum","ParcelTemplateCouriersPleaseEnum","ParcelTemplateDHLeCommerceEnum","ParcelTemplateDPDUKEnum","ParcelTemplateEnumSet","ParcelTemplateFedExEnum","ParcelTemplateUPSEnum","ParcelTemplateUSPSEnum","Parcels","PaymentMethod","Pickup","PickupBase","PickupStatus","PoNumber","PreferredDeliveryTimeframe","Provider","Rate","RatePaginatedList","RecipientType","Refund","RefundPaginatedList","RefundRequestBody","RefundStatus","ResponseMessage","ReturnServiceType","RmaNumber","Security","ServiceGroup","ServiceGroupAccountAndServiceLevel","ServiceGroupCreateRequest","ServiceGroupTypeEnum","ServiceGroupUpdateRequest","ServiceLevel","ServiceLevelAPCPostalEnum","ServiceLevelAPGEnum","ServiceLevelAirterraEnum","ServiceLevelAramexAustraliaEnum","ServiceLevelAsendiaEnum","ServiceLevelAustraliaPostEnum","ServiceLevelBetterTrucksEnum","ServiceLevelCDLEnum","ServiceLevelCanadaPostEnum","ServiceLevelChronopostEnum","ServiceLevelColissimoEnum","ServiceLevelCorreosEspanaEnum","ServiceLevelCouriersPleaseEnum","ServiceLevelDHLExpressEnum","ServiceLevelDHLGermanyEnum","ServiceLevelDHLeCommerceEnum","ServiceLevelDPDDEEnum","ServiceLevelDPDUKEnum","ServiceLevelDeutschePostEnum","ServiceLevelEnumSet","ServiceLevelEvriUKEnum","ServiceLevelFedExEnum","ServiceLevelGLSUSEnum","ServiceLevelGlobegisticsEnum","ServiceLevelJitsuEnum","ServiceLevelLSOEnum","ServiceLevelLasershipEnum","ServiceLevelMondialRelayEnum","ServiceLevelOnTracEnum","ServiceLevelParcelforceEnum","ServiceLevelPostItalianeEnum","ServiceLevelPurolatorEnum","ServiceLevelRoyalMailEnum","ServiceLevelSendleEnum","ServiceLevelSwyftEnum","ServiceLevelUDSEnum","ServiceLevelUPSEnum","ServiceLevelUSPSEnum","ServiceLevelVehoEnum","ServiceLevelWithParent","ServiceLevelePostGlobalEnum","Shipment","ShipmentCreateRequest","ShipmentCreateRequestCustomsDeclaration","ShipmentExtra","ShipmentExtraLasershipAttributesEnum","ShipmentExtraReturnServiceTypeLasershipEnum","ShipmentExtraReturnServiceTypeUPSEnum","ShipmentPaginatedList","ShipmentStatus","ShippoAccount","ShippoAccountPaginatedList","ShippoAccountUpdateRequest","SignatureConfirmation","Status","Track","TrackingStatus","TrackingStatusEnum","TrackingStatusLocationBase","TrackingStatusSubstatus","TracksRequest","Transaction","TransactionCreateRequest","TransactionPaginatedList","TransactionRate","TransactionStatusEnum","Type","UPSConnectExistingOwnAccountParameters","UPSReferenceFields","UserParcelTemplate","UserParcelTemplateCreateRequest","UserParcelTemplateList","UserParcelTemplateUpdateRequest","UserParcelTemplateWithCarrierTemplateCreateRequest","UserParcelTemplateWithoutCarrierTemplateCreateRequest","Webhook","WebhookEventTypeEnum","WebhookPaginatedList","WebhookPayload","WebhookPayloadBatch","WebhookPayloadTrack","WebhookPayloadTransaction","WebhookUpdateRequest","WeightUnitEnum"]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import requests as requests_http
from .sdkconfiguration import SDKConfiguration
from shippo import utils
from shippo._hooks import AfterErrorContext, AfterSuccessContext, BeforeRequestContext, HookContext
from shippo.models import components, errors, operations
from typing import Optional

class RatesAtCheckout:
    r"""Rates at checkout is a tool for merchants to display up-to-date shipping estimates based on what's in their customers cart and where they’re shipping to.
    Merchants set up curated shipping options for customers in the checkout flow based on data in the shopping cart. The request must include the **to** address and item information. Optional fields are the **from** address and package information. If the optional fields are not included, the service will use the default address and/or package configured for rates at checkout. The response is a list of shipping options based on the Service Group configuration.
    (see <a href=\"#tag/Service-Groups\">Service Group configuration</a> for details).
    <SchemaDefinition schemaRef=\"#/components/schemas/LiveRate\"/>


    # Default Parcel Template
    Assign one of your user parcel templates to be the default used when generating Live Rates. This template will be used by default when generating Live Rates, unless you explicitly provide a parcel in the Live Rates request.
    <SchemaDefinition schemaRef=\"#/components/schemas/UserParcelTemplate\"/>
    """
    sdk_configuration: SDKConfiguration

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        self.sdk_configuration = sdk_config
        
    
    
    def create(self, request: components.LiveRateCreateRequest) -> components.LiveRatePaginatedList:
        r"""Generate a live rates request
        Initiates a live rates request. Include either the object ID for
        an existing address record or a fully formed address object when entering
        an address value. You can also enter the object ID of an existing user parcel
        template or a fully formed user parcel template object as the parcel value.
        """
        hook_ctx = HookContext(operation_id='CreateLiveRate', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        _globals = operations.CreateLiveRateGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/live-rates'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        req_content_type, data, form = utils.serialize_request_body(request, components.LiveRateCreateRequest, "request", False, False, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        if data is None and form is None:
            raise Exception('request body is required')
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('POST', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.LiveRatePaginatedList])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def get_default_parcel_template(self) -> components.DefaultParcelTemplate:
        r"""Show current default parcel template
        Retrieve and display the currently configured default parcel template for live rates.
        """
        hook_ctx = HookContext(operation_id='GetDefaultParcelTemplate', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.GetDefaultParcelTemplateRequest(
        )
        
        _globals = operations.GetDefaultParcelTemplateGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/live-rates/settings/parcel-template'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('GET', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.DefaultParcelTemplate])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def update_default_parcel_template(self, object_id: Optional[str] = None) -> components.DefaultParcelTemplate:
        r"""Update default parcel template
        Update the currently configured default parcel template for live rates. The object_id in the request payload should identify the user parcel template to be the new default.
        """
        hook_ctx = HookContext(operation_id='UpdateDefaultParcelTemplate', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = components.DefaultParcelTemplateUpdateRequest(
            object_id=object_id,
        )
        
        _globals = operations.UpdateDefaultParcelTemplateGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/live-rates/settings/parcel-template'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        req_content_type, data, form = utils.serialize_request_body(request, Optional[components.DefaultParcelTemplateUpdateRequest], "request", False, True, 'json')
        if req_content_type is not None and req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('PUT', url, params=query_params, data=data, files=form, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 200:
            # pylint: disable=no-else-return
            if utils.match_content_type(http_res.headers.get('Content-Type') or '', 'application/json'):                
                out = utils.unmarshal_json(http_res.text, Optional[components.DefaultParcelTemplate])
                return out
            
            content_type = http_res.headers.get('Content-Type')
            raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    
    
    def delete_default_parcel_template(self):
        r"""Clear current default parcel template
        Clears the currently configured default parcel template for live rates.
        """
        hook_ctx = HookContext(operation_id='DeleteDefaultParcelTemplate', oauth2_scopes=[], security_source=self.sdk_configuration.security)
        request = operations.DeleteDefaultParcelTemplateRequest(
        )
        
        _globals = operations.DeleteDefaultParcelTemplateGlobals(
            shippo_api_version=self.sdk_configuration.globals.shippo_api_version,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/live-rates/settings/parcel-template'
        
        if callable(self.sdk_configuration.security):
            headers, query_params = utils.get_security(self.sdk_configuration.security())
        else:
            headers, query_params = utils.get_security(self.sdk_configuration.security)
        
        headers = { **utils.get_headers(request, _globals), **headers }
        headers['Accept'] = '*/*'
        headers['user-agent'] = self.sdk_configuration.user_agent
        client = self.sdk_configuration.client
        
        try:
            req = client.prepare_request(requests_http.Request('DELETE', url, params=query_params, headers=headers))
            req = self.sdk_configuration.get_hooks().before_request(BeforeRequestContext(hook_ctx), req)
            http_res = client.send(req)
        except Exception as e:
            _, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), None, e)
            if e is not None:
                raise e

        if utils.match_status_codes(['400','4XX','5XX'], http_res.status_code):
            result, e = self.sdk_configuration.get_hooks().after_error(AfterErrorContext(hook_ctx), http_res, None)
            if e is not None:
                raise e
            if result is not None:
                http_res = result
        else:
            http_res = self.sdk_configuration.get_hooks().after_success(AfterSuccessContext(hook_ctx), http_res)
            
        
        
        
        if http_res.status_code == 204:
            pass
        elif http_res.status_code == 400 or http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        else:
            raise errors.SDKError('unknown status code received', http_res.status_code, http_res.text, http_res)

    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .customstaxidentification import CustomsTaxIdentification
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CustomsExporterIdentification:
    r"""Additional exporter identification that may be required to ship in certain countries"""
    eori_number: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('eori_number'), 'exclude': lambda f: f is None }})
    r"""Economic Operators' Registration and Identification (EORI) number. Must start with a 2 character
    country code followed by a 6-17 character alphanumeric identifier (e.g. PL1234567890ABCDE).
    <a href=\"https://ec.europa.eu/taxation_customs/business/customs-procedures/general-overview/economic-operators-registration-identification-number-eori_en\">More information on EORI.</a>
    """
    tax_id: Optional[CustomsTaxIdentification] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('tax_id'), 'exclude': lambda f: f is None }})
    r"""Tax identification that may be required to ship in certain countries. Typically used to assess duties on
    goods that are crossing a border.
    """
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from .weightunitenum import WeightUnitEnum
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class LineItemBase:
    currency: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('currency'), 'exclude': lambda f: f is None }})
    r"""Currency of the <code>total_price</code> amount."""
    manufacture_country: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('manufacture_country'), 'exclude': lambda f: f is None }})
    r"""Country the item was manufactured in. In the Shippo dashboard, this value will be used ot pre-fill the customs declaration when creating a label for this order."""
    max_delivery_time: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('max_delivery_time'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""The date and time this item needs to be delivered by, i.e. by when the carrier delivers it to the buyer.
    This value is used by some platforms such as eBay to measure a seller's shipping time and performance. 
    It will be displayed in the Shippo dashboard.
    """
    max_ship_time: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('max_ship_time'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""The date and time this item needs to be fulfilled by, i.e. by when the shipping label needs to be
    created and handed over to the carrier. This value is used by some platforms such as eBay to measure 
    a seller's handling time and performance. It will be displayed in the Shippo dashboard.
    """
    quantity: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('quantity'), 'exclude': lambda f: f is None }})
    r"""The quantity of this item in this order."""
    sku: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('sku'), 'exclude': lambda f: f is None }})
    r"""The stock keeping unit value of this item."""
    title: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('title'), 'exclude': lambda f: f is None }})
    r"""Title of the line item."""
    total_price: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('total_price'), 'exclude': lambda f: f is None }})
    r"""Total price paid by the buyer for this item (or these items, if quantity > 1)."""
    variant_title: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('variant_title'), 'exclude': lambda f: f is None }})
    r"""A variant is a specific variation of an item (e.g. `size M` or `color blue`).
    Variants might be exposed as a separate resource in the future too. 
    Currently the variant title is a free text field describing the variant.
    """
    weight: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('weight'), 'exclude': lambda f: f is None }})
    r"""Total weight of this/these item(s). Instead of specifying the weight of all items,
    you can also set the <code>total_weight</code> value of the order object.
    """
    weight_unit: Optional[WeightUnitEnum] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('weight_unit'), 'exclude': lambda f: f is None }})
    r"""The unit used for weight."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ParcelTemplateCouriersPleaseEnum(str, Enum):
    r"""|Token | Name | Dimensions|
    |:---|:---|:---|
    | couriersplease_500g_satchel | 500g Satchel | 22.00 x 33.50 x 0.10 cm|
    | couriersplease_1kg_satchel | 1kg Satchel | 28.00 x 35.00 x 0.10 cm|
    | couriersplease_3kg_satchel | 3kg Satchel | 34.00 x 42.00 x 0.10 cm|
    | couriersplease_5kg_satchel | 5kg Satchel | 43.70 x 59.70 x 0.10 cm|
    """
    COURIERSPLEASE_500G_SATCHEL = 'couriersplease_500g_satchel'
    COURIERSPLEASE_1KG_SATCHEL = 'couriersplease_1kg_satchel'
    COURIERSPLEASE_3KG_SATCHEL = 'couriersplease_3kg_satchel'
    COURIERSPLEASE_5KG_SATCHEL = 'couriersplease_5kg_satchel'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .weightunitenum import WeightUnitEnum
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class UserParcelTemplateWithCarrierTemplateCreateRequest:
    template: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('template'), 'exclude': lambda f: f is None }})
    r"""The object representing the carrier parcel template"""
    weight: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('weight'), 'exclude': lambda f: f is None }})
    r"""The weight of the package, in units specified by the weight_unit attribute."""
    weight_unit: Optional[WeightUnitEnum] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('weight_unit'), 'exclude': lambda f: f is None }})
    r"""The unit used for weight."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelSwyftEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | same_day | Next Day|
    | next_day | Next Day|
    """
    SAME_DAY = 'same_day'
    NEXT_DAY = 'next_day'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelUSPSEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | usps_priority | Priority Mail|
    | usps_priority_express | Priority Mail Express|
    | usps_media_mail | Media Mail, only for existing Shippo customers with grandfathered Media Mail option.|
    | usps_priority_mail_international | Priority Mail International|
    | usps_priority_mail_express_international | Priority Mail Express International|
    | usps_first_class_package_international_service | First Class Package International|
    | usps_ground_advantage | Ground Advantage |
    """
    USPS_PRIORITY = 'usps_priority'
    USPS_PRIORITY_EXPRESS = 'usps_priority_express'
    USPS_MEDIA_MAIL = 'usps_media_mail'
    USPS_PRIORITY_MAIL_INTERNATIONAL = 'usps_priority_mail_international'
    USPS_PRIORITY_MAIL_EXPRESS_INTERNATIONAL = 'usps_priority_mail_express_international'
    USPS_FIRST_CLASS_PACKAGE_INTERNATIONAL_SERVICE = 'usps_first_class_package_international_service'
    USPS_GROUND_ADVANTAGE = 'usps_ground_advantage'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .addressvalidationresultsmessage import AddressValidationResultsMessage
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import List, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class AddressValidationResults:
    r"""Object that contains information regarding if an address had been validated or not. Also contains any messages
    generated during validation. Children keys are <code>is_valid</code>(boolean) and <code>messages</code>(array).
    """
    is_valid: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('is_valid'), 'exclude': lambda f: f is None }})
    messages: Optional[List[AddressValidationResultsMessage]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('messages'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class OrderStatusEnum(str, Enum):
    r"""Current state of the order. See the <a href=\\"https://docs.goshippo.com/docs/orders/orders/\\">orders tutorial</a>
    for the logic of how the status is handled.
    """
    UNKNOWN = 'UNKNOWN'
    AWAITPAY = 'AWAITPAY'
    PAID = 'PAID'
    REFUNDED = 'REFUNDED'
    CANCELLED = 'CANCELLED'
    PARTIALLY_FULFILLED = 'PARTIALLY_FULFILLED'
    SHIPPED = 'SHIPPED'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .parcelextra import ParcelExtra
from .parceltemplateenumset import ParcelTemplateEnumSet
from .weightunitenum import WeightUnitEnum
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class ParcelCreateFromTemplateRequest:
    mass_unit: WeightUnitEnum = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('mass_unit') }})
    r"""The unit used for weight."""
    weight: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('weight') }})
    r"""Weight of the parcel. Up to six digits in front and four digits after the decimal separator are accepted."""
    template: ParcelTemplateEnumSet = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('template') }})
    r"""If template is passed, `length`, `width`, `height`, and `distance_unit` are not required"""
    extra: Optional[ParcelExtra] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('extra'), 'exclude': lambda f: f is None }})
    r"""An object holding optional extra services to be requested for each parcel in a multi-piece shipment.
    See the <a href=\"#section/Parcel-Extras\">Parcel Extra table below</a> for all available services.
    """
    metadata: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ObjectStateEnum(str, Enum):
    r"""Indicates the validity of the enclosing object"""
    VALID = 'VALID'
    INVALID = 'INVALID'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class AddressValidationResultsMessageSourceEnum(str, Enum):
    SHIPPO_ADDRESS_VALIDATOR = 'Shippo Address Validator'
    UPS = 'UPS'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ParcelTemplateFedExEnum(str, Enum):
    r"""|Token | Name | Dimensions|
    |:---|:---|:---|
    | FedEx_Box_10kg | FedEx® 10kg Box | 15.81 x 12.94 x 10.19 in|
    | FedEx_Box_25kg | FedEx® 25kg Box | 54.80 x 42.10 x 33.50 in|
    | FedEx_Box_Extra_Large_1 | FedEx® Extra Large Box (X1) | 11.88 x 11.00 x 10.75 in|
    | FedEx_Box_Extra_Large_2 | FedEx® Extra Large Box (X2) | 15.75 x 14.13 x 6.00 in|
    | FedEx_Box_Large_1 | FedEx® Large Box (L1) | 17.50 x 12.38 x 3.00 in|
    | FedEx_Box_Large_2 | FedEx® Large Box (L2) | 11.25 x 8.75 x 7.75 in|
    | FedEx_Box_Medium_1 | FedEx® Medium Box (M1) | 13.25 x 11.50 x 2.38 in|
    | FedEx_Box_Medium_2 | FedEx® Medium Box (M2) | 11.25 x 8.75 x 4.38 in|
    | FedEx_Box_Small_1 | FedEx® Small Box (S1) | 12.38 x 10.88 x 1.50 in|
    | FedEx_Box_Small_2 | FedEx® Small Box (S2) | 11.25 x 8.75 x 4.38 in|
    | FedEx_Envelope | FedEx® Envelope | 12.50 x 9.50 x 0.80 in|
    | FedEx_Padded_Pak | FedEx® Padded Pak | 11.75 x 14.75 x 2.00 in|
    | FedEx_Pak_1 | FedEx® Large Pak | 15.50 x 12.00 x 0.80 in|
    | FedEx_Pak_2 | FedEx® Small Pak | 12.75 x 10.25 x 0.80 in|
    | FedEx_Tube | FedEx® Tube | 38.00 x 6.00 x 6.00 in|
    | FedEx_XL_Pak | FedEx® Extra Large Pak | 17.50 x 20.75 x 2.00 in|
    """
    FED_EX_BOX_10KG = 'FedEx_Box_10kg'
    FED_EX_BOX_25KG = 'FedEx_Box_25kg'
    FED_EX_BOX_EXTRA_LARGE_1 = 'FedEx_Box_Extra_Large_1'
    FED_EX_BOX_EXTRA_LARGE_2 = 'FedEx_Box_Extra_Large_2'
    FED_EX_BOX_LARGE_1 = 'FedEx_Box_Large_1'
    FED_EX_BOX_LARGE_2 = 'FedEx_Box_Large_2'
    FED_EX_BOX_MEDIUM_1 = 'FedEx_Box_Medium_1'
    FED_EX_BOX_MEDIUM_2 = 'FedEx_Box_Medium_2'
    FED_EX_BOX_SMALL_1 = 'FedEx_Box_Small_1'
    FED_EX_BOX_SMALL_2 = 'FedEx_Box_Small_2'
    FED_EX_ENVELOPE = 'FedEx_Envelope'
    FED_EX_PADDED_PAK = 'FedEx_Padded_Pak'
    FED_EX_PAK_1 = 'FedEx_Pak_1'
    FED_EX_PAK_2 = 'FedEx_Pak_2'
    FED_EX_TUBE = 'FedEx_Tube'
    FED_EX_XL_PAK = 'FedEx_XL_Pak'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelDHLGermanyEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | dhl_germany_europaket | DHL Germany Europaket|
    | dhl_germany_paket | DHL Germany Paket|
    | dhl_germany_paket_connect | DHL Germany Paket Connect|
    | dhl_germany_paket_international | DHL Germany Paket International|
    | dhl_germany_paket_priority | DHL Germany Paket Priority|
    | dhl_germany_paket_sameday | DHL Germany Paket Sameday|
    """
    DHL_GERMANY_EUROPAKET = 'dhl_germany_europaket'
    DHL_GERMANY_PAKET = 'dhl_germany_paket'
    DHL_GERMANY_PAKET_CONNECT = 'dhl_germany_paket_connect'
    DHL_GERMANY_PAKET_INTERNATIONAL = 'dhl_germany_paket_international'
    DHL_GERMANY_PAKET_PRIORITY = 'dhl_germany_paket_priority'
    DHL_GERMANY_PAKET_SAMEDAY = 'dhl_germany_paket_sameday'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelCDLEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | cdl_next_day | Next Day|
    """
    CDL_NEXT_DAY = 'cdl_next_day'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class ResponseMessage:
    r"""Message returned with supporting information from a request. In some cases this can be an error message,
    for example a timeout from a carrier. If available, the origin of the message is displayed in `source`.
    """
    source: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('source'), 'exclude': lambda f: f is None }})
    r"""Origin of message"""
    code: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('code'), 'exclude': lambda f: f is None }})
    r"""Classification of message"""
    text: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('text'), 'exclude': lambda f: f is None }})
    r"""Message content"""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelDHLeCommerceEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | dhl_ecommerce_marketing_parcel_expedited | Marketing Parcel Expedited|
    | dhl_ecommerce_globalmail_business_ipa | GlobalMail Business IPA|
    | dhl_ecommerce_parcel_international_direct | Parcel International Direct|
    | dhl_ecommerce_parcels_expedited_max | Parcels Expedited Max|
    | dhl_ecommerce_bpm_ground | Bounded Printed Matter Ground|
    | dhl_ecommerce_priority_expedited | Priority Expedited|
    | dhl_ecommerce_globalmail_packet_ipa | GlobalMail Packet IPA|
    | dhl_ecommerce_globalmail_packet_isal | GlobalMail Packet ISAL|
    | dhl_ecommerce_easy_return_plus | Easy Return Plus|
    | dhl_ecommerce_marketing_parcel_ground | Marketing Parcel Ground|
    | dhl_ecommerce_first_class_parcel_expedited | First Class Parcel Expedited|
    | dhl_ecommerce_globalmail_business_priority | GlobalMail Business Priority|
    | dhl_ecommerce_parcels_expedited | Parcels Expedited|
    | dhl_ecommerce_globalmail_business_isal | GlobalMail Business ISAL|
    | dhl_ecommerce_parcel_plus_expedited_max | Parcel Plus Expedited Max|
    | dhl_ecommerce_globalmail_packet_plus | GlobalMail Packet Plus|
    | dhl_ecommerce_parcels_ground | Parcels Ground|
    | dhl_ecommerce_expedited | Expedited|
    | dhl_ecommerce_parcel_plus_ground | Parcel Plus Ground|
    | dhl_ecommerce_parcel_international_standard | Parcel International Standard|
    | dhl_ecommerce_bpm_expedited | Bounded Printed Matter Expedited|
    | dhl_ecommerce_parcel_international_expedited | Parcel International Expedited|
    | dhl_ecommerce_globalmail_packet_priority | GlobalMail Packet Priority|
    | dhl_ecommerce_easy_return_light | Easy Return Light|
    | dhl_ecommerce_parcel_plus_expedited | Parcel Plus Expedited|
    | dhl_ecommerce_globalmail_business_standard | GlobalMail Packet Standard|
    | dhl_ecommerce_ground | Ground|
    | dhl_ecommerce_globalmail_packet_standard | GlobalMail Business Standard|
    """
    DHL_ECOMMERCE_MARKETING_PARCEL_EXPEDITED = 'dhl_ecommerce_marketing_parcel_expedited'
    DHL_ECOMMERCE_GLOBALMAIL_BUSINESS_IPA = 'dhl_ecommerce_globalmail_business_ipa'
    DHL_ECOMMERCE_PARCEL_INTERNATIONAL_DIRECT = 'dhl_ecommerce_parcel_international_direct'
    DHL_ECOMMERCE_PARCELS_EXPEDITED_MAX = 'dhl_ecommerce_parcels_expedited_max'
    DHL_ECOMMERCE_BPM_GROUND = 'dhl_ecommerce_bpm_ground'
    DHL_ECOMMERCE_PRIORITY_EXPEDITED = 'dhl_ecommerce_priority_expedited'
    DHL_ECOMMERCE_GLOBALMAIL_PACKET_IPA = 'dhl_ecommerce_globalmail_packet_ipa'
    DHL_ECOMMERCE_GLOBALMAIL_PACKET_ISAL = 'dhl_ecommerce_globalmail_packet_isal'
    DHL_ECOMMERCE_EASY_RETURN_PLUS = 'dhl_ecommerce_easy_return_plus'
    DHL_ECOMMERCE_MARKETING_PARCEL_GROUND = 'dhl_ecommerce_marketing_parcel_ground'
    DHL_ECOMMERCE_FIRST_CLASS_PARCEL_EXPEDITED = 'dhl_ecommerce_first_class_parcel_expedited'
    DHL_ECOMMERCE_GLOBALMAIL_BUSINESS_PRIORITY = 'dhl_ecommerce_globalmail_business_priority'
    DHL_ECOMMERCE_PARCELS_EXPEDITED = 'dhl_ecommerce_parcels_expedited'
    DHL_ECOMMERCE_GLOBALMAIL_BUSINESS_ISAL = 'dhl_ecommerce_globalmail_business_isal'
    DHL_ECOMMERCE_PARCEL_PLUS_EXPEDITED_MAX = 'dhl_ecommerce_parcel_plus_expedited_max'
    DHL_ECOMMERCE_GLOBALMAIL_PACKET_PLUS = 'dhl_ecommerce_globalmail_packet_plus'
    DHL_ECOMMERCE_PARCELS_GROUND = 'dhl_ecommerce_parcels_ground'
    DHL_ECOMMERCE_EXPEDITED = 'dhl_ecommerce_expedited'
    DHL_ECOMMERCE_PARCEL_PLUS_GROUND = 'dhl_ecommerce_parcel_plus_ground'
    DHL_ECOMMERCE_PARCEL_INTERNATIONAL_STANDARD = 'dhl_ecommerce_parcel_international_standard'
    DHL_ECOMMERCE_BPM_EXPEDITED = 'dhl_ecommerce_bpm_expedited'
    DHL_ECOMMERCE_PARCEL_INTERNATIONAL_EXPEDITED = 'dhl_ecommerce_parcel_international_expedited'
    DHL_ECOMMERCE_GLOBALMAIL_PACKET_PRIORITY = 'dhl_ecommerce_globalmail_packet_priority'
    DHL_ECOMMERCE_EASY_RETURN_LIGHT = 'dhl_ecommerce_easy_return_light'
    DHL_ECOMMERCE_PARCEL_PLUS_EXPEDITED = 'dhl_ecommerce_parcel_plus_expedited'
    DHL_ECOMMERCE_GLOBALMAIL_BUSINESS_STANDARD = 'dhl_ecommerce_globalmail_business_standard'
    DHL_ECOMMERCE_GROUND = 'dhl_ecommerce_ground'
    DHL_ECOMMERCE_GLOBALMAIL_PACKET_STANDARD = 'dhl_ecommerce_globalmail_packet_standard'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ServiceLevelLSOEnum(str, Enum):
    r"""|Token | Service name|
    |:---|:---|
    | lso_ground | Ground|
    | lso_economy_next_day | Economy Next Day|
    | lso_saturday_delivery | Saturday Delivery|
    | lso_2nd_day | 2nd Day|
    | lso_priority_next_day | Priority Next Day|
    | lso_early_overnight | Early Overnight|
    """
    LSO_GROUND = 'lso_ground'
    LSO_ECONOMY_NEXT_DAY = 'lso_economy_next_day'
    LSO_SATURDAY_DELIVERY = 'lso_saturday_delivery'
    LSO_2ND_DAY = 'lso_2nd_day'
    LSO_PRIORITY_NEXT_DAY = 'lso_priority_next_day'
    LSO_EARLY_OVERNIGHT = 'lso_early_overnight'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CarrierAccountUPSCreateRequestParameters:
    billing_address_city: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('billing_address_city') }})
    billing_address_country_iso2: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('billing_address_country_iso2') }})
    billing_address_state: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('billing_address_state') }})
    billing_address_street1: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('billing_address_street1') }})
    billing_address_zip: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('billing_address_zip') }})
    pickup_address_city: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('pickup_address_city') }})
    r"""User's pickup address city."""
    pickup_address_country_iso2: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('pickup_address_country_iso2') }})
    r"""User's pickup street 1."""
    pickup_address_state: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('pickup_address_state') }})
    r"""User's pickup address state."""
    pickup_address_street1: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('pickup_address_street1') }})
    r"""User's pickup address street 1."""
    pickup_address_zip: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('pickup_address_zip') }})
    r"""User's pickup address zip code."""
    ups_agreements: bool = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('ups_agreements') }})
    r"""Whether the user agrees to the UPS terms and conditions or not. Error 400 will be returned if passed in as false"""
    billing_address_street2: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('billing_address_street2'), 'exclude': lambda f: f is None }})
    r"""Empty string acceptable for billing_address_street2"""
    company: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('company'), 'exclude': lambda f: f is None }})
    r"""Company name. Full name is acceptable in this field if the user has no company name"""
    email: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('email'), 'exclude': lambda f: f is None }})
    full_name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('full_name'), 'exclude': lambda f: f is None }})
    phone: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('phone'), 'exclude': lambda f: f is None }})
    r"""Needs to be a valid phone number and cannot be null"""
    pickup_address_same_as_billing_address: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('pickup_address_same_as_billing_address'), 'exclude': lambda f: f is None }})
    pickup_address_street2: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('pickup_address_street2'), 'exclude': lambda f: f is None }})
    r"""User's pickup street 2."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ParcelTemplateDPDUKEnum(str, Enum):
    r"""|Token | Name | Dimensions|
    |:---|:---|:---|
    | DPD_UK_Express_Pak| DPD UK Express Pak | 530.00 x 400.00 x 100.00 mm|
    """
    DPD_UK_EXPRESS_PAK = 'DPD_UK_Express_Pak'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .servicelevelairterraenum import ServiceLevelAirterraEnum
from .servicelevelapcpostalenum import ServiceLevelAPCPostalEnum
from .servicelevelapgenum import ServiceLevelAPGEnum
from .servicelevelaramexaustraliaenum import ServiceLevelAramexAustraliaEnum
from .servicelevelasendiaenum import ServiceLevelAsendiaEnum
from .servicelevelaustraliapostenum import ServiceLevelAustraliaPostEnum
from .servicelevelbettertrucksenum import ServiceLevelBetterTrucksEnum
from .servicelevelcanadapostenum import ServiceLevelCanadaPostEnum
from .servicelevelcdlenum import ServiceLevelCDLEnum
from .servicelevelchronopostenum import ServiceLevelChronopostEnum
from .servicelevelcolissimoenum import ServiceLevelColissimoEnum
from .servicelevelcorreosespanaenum import ServiceLevelCorreosEspanaEnum
from .servicelevelcourierspleaseenum import ServiceLevelCouriersPleaseEnum
from .serviceleveldeutschepostenum import ServiceLevelDeutschePostEnum
from .serviceleveldhlecommerceenum import ServiceLevelDHLeCommerceEnum
from .serviceleveldhlexpressenum import ServiceLevelDHLExpressEnum
from .serviceleveldhlgermanyenum import ServiceLevelDHLGermanyEnum
from .serviceleveldpddeenum import ServiceLevelDPDDEEnum
from .serviceleveldpdukenum import ServiceLevelDPDUKEnum
from .servicelevelepostglobalenum import ServiceLevelePostGlobalEnum
from .servicelevelevriukenum import ServiceLevelEvriUKEnum
from .servicelevelfedexenum import ServiceLevelFedExEnum
from .servicelevelglobegisticsenum import ServiceLevelGlobegisticsEnum
from .servicelevelglsusenum import ServiceLevelGLSUSEnum
from .serviceleveljitsuenum import ServiceLevelJitsuEnum
from .servicelevellasershipenum import ServiceLevelLasershipEnum
from .servicelevellsoenum import ServiceLevelLSOEnum
from .servicelevelmondialrelayenum import ServiceLevelMondialRelayEnum
from .servicelevelontracenum import ServiceLevelOnTracEnum
from .servicelevelparcelforceenum import ServiceLevelParcelforceEnum
from .servicelevelpostitalianeenum import ServiceLevelPostItalianeEnum
from .servicelevelpurolatorenum import ServiceLevelPurolatorEnum
from .servicelevelroyalmailenum import ServiceLevelRoyalMailEnum
from .servicelevelsendleenum import ServiceLevelSendleEnum
from .servicelevelswyftenum import ServiceLevelSwyftEnum
from .serviceleveludsenum import ServiceLevelUDSEnum
from .servicelevelupsenum import ServiceLevelUPSEnum
from .serviceleveluspsenum import ServiceLevelUSPSEnum
from .servicelevelvehoenum import ServiceLevelVehoEnum
from typing import Union

ServiceLevelEnumSet = Union[ServiceLevelUSPSEnum, ServiceLevelFedExEnum, ServiceLevelUPSEnum, ServiceLevelAirterraEnum, ServiceLevelAPCPostalEnum, ServiceLevelAsendiaEnum, ServiceLevelAustraliaPostEnum, ServiceLevelAPGEnum, ServiceLevelBetterTrucksEnum, ServiceLevelCanadaPostEnum, ServiceLevelCDLEnum, ServiceLevelChronopostEnum, ServiceLevelCouriersPleaseEnum, ServiceLevelCorreosEspanaEnum, ServiceLevelColissimoEnum, ServiceLevelPurolatorEnum, ServiceLevelDHLExpressEnum, ServiceLevelDHLeCommerceEnum, ServiceLevelDHLGermanyEnum, ServiceLevelDPDDEEnum, ServiceLevelDPDUKEnum, ServiceLevelDeutschePostEnum, ServiceLevelAramexAustraliaEnum, ServiceLevelGlobegisticsEnum, ServiceLevelGLSUSEnum, ServiceLevelLSOEnum, ServiceLevelMondialRelayEnum, ServiceLevelParcelforceEnum, ServiceLevelPostItalianeEnum, ServiceLevelePostGlobalEnum, ServiceLevelRoyalMailEnum, ServiceLevelSendleEnum, ServiceLevelOnTracEnum, ServiceLevelJitsuEnum, ServiceLevelLasershipEnum, ServiceLevelEvriUKEnum, ServiceLevelUDSEnum, ServiceLevelVehoEnum, ServiceLevelSwyftEnum]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from .corerate import CoreRate
from .labelfiletypeenum import LabelFileTypeEnum
from .objectstateenum import ObjectStateEnum
from .responsemessage import ResponseMessage
from .trackingstatusenum import TrackingStatusEnum
from .transactionstatusenum import TransactionStatusEnum
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from shippo import utils
from typing import List, Optional, Union


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CreatedBy:
    r"""An object with details about the user who created the Transaction (purchased the label).
    A value will be returned only for Transactions that can be associated with a specific user, e.g. when a logged-in
    user purchases a label via the Shippo Web application; but not for Transactions purchased e.g. via the API using a ShippoToken,
    which is associated with the account but not any specific user.
    """
    first_name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('first_name'), 'exclude': lambda f: f is None }})
    last_name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('last_name'), 'exclude': lambda f: f is None }})
    username: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('username'), 'exclude': lambda f: f is None }})
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Transaction:
    UNSET='__SPEAKEASY_UNSET__'
    commercial_invoice_url: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('commercial_invoice_url'), 'exclude': lambda f: f is None }})
    r"""A URL pointing to the commercial invoice as a 8.5x11 inch PDF file.
    A value will only be returned if the Transactions has been processed successfully and if the shipment is international.
    """
    created_by: Optional[CreatedBy] = dataclasses.field(default=UNSET, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('created_by'), 'exclude': lambda f: f is Transaction.UNSET }})
    eta: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('eta'), 'exclude': lambda f: f is None }})
    r"""The estimated time of arrival according to the carrier."""
    label_file_type: Optional[LabelFileTypeEnum] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('label_file_type'), 'exclude': lambda f: f is None }})
    r"""Print format of the <a href=\\"https://docs.goshippo.com/docs/shipments/shippinglabelsizes/\\">label</a>. If empty, will use the default format set from
    <a href=\"https://apps.goshippo.com/settings/labels\">the Shippo dashboard.</a>
    """
    label_url: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('label_url'), 'exclude': lambda f: f is None }})
    r"""A URL pointing directly to the label in the format you've set in your settings.
    A value will only be returned if the Transactions has been processed successfully.
    """
    messages: Optional[List[ResponseMessage]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('messages'), 'exclude': lambda f: f is None }})
    metadata: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    r"""A string of up to 100 characters that can be filled with any additional information you want to
    attach to the object.
    """
    object_created: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_created'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Date and time of Transaction creation."""
    object_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_id'), 'exclude': lambda f: f is None }})
    r"""Unique identifier of the given Transaction object."""
    object_owner: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_owner'), 'exclude': lambda f: f is None }})
    r"""Username of the user who created the Transaction object."""
    object_state: Optional[ObjectStateEnum] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_state'), 'exclude': lambda f: f is None }})
    r"""Indicates the validity of the enclosing object"""
    object_updated: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_updated'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Date and time of last Transaction update."""
    parcel: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('parcel'), 'exclude': lambda f: f is None }})
    r"""Object ID of the Parcel object that is being shipped."""
    qr_code_url: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('qr_code_url'), 'exclude': lambda f: f is None }})
    r"""A URL pointing directly to the QR code in PNG format.
    A value will only be returned if requested using qr_code_requested flag and the carrier provides such an option.
    """
    rate: Optional[TransactionRate] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('rate'), 'exclude': lambda f: f is None }})
    r"""ID of the Rate object for which a Label has to be obtained.
    If you purchase a label by calling the transaction endpoint without a rate (instalabel), 
    this field will be a simplified Rate object in the Transaction model returned from the POST request.
    </br>Note, only rates less than 7 days old can be purchased to ensure up-to-date pricing.
    """
    status: Optional[TransactionStatusEnum] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status'), 'exclude': lambda f: f is None }})
    r"""Indicates the status of the Transaction."""
    test: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('test'), 'exclude': lambda f: f is None }})
    r"""Indicates whether the object has been created in test mode."""
    tracking_number: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('tracking_number'), 'exclude': lambda f: f is None }})
    r"""The carrier-specific tracking number that can be used to track the Shipment.
    A value will only be returned if the Rate is for a trackable Shipment and if the Transactions has been processed successfully.
    """
    tracking_status: Optional[TrackingStatusEnum] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('tracking_status'), 'exclude': lambda f: f is None }})
    r"""Indicates the high level status of the shipment."""
    tracking_url_provider: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('tracking_url_provider'), 'exclude': lambda f: f is None }})
    r"""A link to track this item on the carrier-provided tracking website.
    A value will only be returned if tracking is available and the carrier provides such a service.
    """
    


TransactionRate = Union[CoreRate, str]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from .customsexporteridentification import CustomsExporterIdentification
from .customsinvoicedcharges import CustomsInvoicedCharges
from .objectstateenum import ObjectStateEnum
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from enum import Enum
from shippo import utils
from typing import List, Optional


class CustomsDeclarationType(str, Enum):
    r"""Party to be billed for duties."""
    SENDER = 'SENDER'
    RECIPIENT = 'RECIPIENT'
    THIRD_PARTY = 'THIRD_PARTY'


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CustomsDeclarationAddress:
    name: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name'), 'exclude': lambda f: f is None }})
    r"""Name of the party to be billed for duties."""
    zip: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('zip'), 'exclude': lambda f: f is None }})
    r"""Postal code of the party to be billed for duties."""
    country: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('country'), 'exclude': lambda f: f is None }})
    r"""Country ISO code of account number to be billed."""
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CustomsDeclarationDutiesPayor:
    r"""Specifies who will pay the duties for the shipment. Only accepted for FedEx shipments."""
    account: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('account'), 'exclude': lambda f: f is None }})
    r"""Account number to be billed for duties."""
    type: Optional[CustomsDeclarationType] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type'), 'exclude': lambda f: f is None }})
    r"""Party to be billed for duties."""
    address: Optional[CustomsDeclarationAddress] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('address'), 'exclude': lambda f: f is None }})
    



@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class CustomsDeclaration:
    certify: bool = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('certify') }})
    r"""Expresses that the certify_signer has provided all information of this customs declaration truthfully."""
    certify_signer: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('certify_signer') }})
    r"""Name of the person who created the customs declaration and is responsible for the validity of all
    information provided.
    """
    contents_type: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('contents_type') }})
    r"""Type of goods of the shipment.
    Allowed values available <a href=\"#tag/Customs-Declaration-Contents-Type\">here</a>
    """
    items: List[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('items') }})
    r"""Distinct Parcel content items as Customs Items object_ids."""
    non_delivery_option: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('non_delivery_option') }})
    r"""Indicates how the carrier should proceed in case the shipment can't be delivered.
    Allowed values available <a href=\"#tag/Customs-Declaration-Non-Delivery-Option\">here</a>
    """
    aes_itn: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('aes_itn'), 'exclude': lambda f: f is None }})
    r"""**required if eel_pfc is `AES_ITN`**<br>
    AES / ITN reference of the shipment.
    """
    b13a_filing_option: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('b13a_filing_option'), 'exclude': lambda f: f is None }})
    r"""B13A Option details are obtained by filing a B13A Canada Export Declaration via the Canadian Export Reporting System (CERS).
    <a href=\"https://www.cbsa-asfc.gc.ca/services/export/guide-eng.html\" target=\"_blank\" rel=\"noopener noreferrer\"> More information on reporting commercial exports from Canada. </a>
    Allowed values available <a href=\"#tag/Customs-Declaration-B13A-Filing-Option\">here</a>
    """
    b13a_number: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('b13a_number'), 'exclude': lambda f: f is None }})
    r"""**must be provided if and only if b13a_filing_option is provided**<br>
    Represents:<br> the Proof of Report (POR) Number when b13a_filing_option is `FILED_ELECTRONICALLY`;<br> 
    the Summary ID Number when b13a_filing_option is `SUMMARY_REPORTING`;<br> 
    or the Exemption Number when b13a_filing_option is `NOT_REQUIRED`.
    """
    certificate: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('certificate'), 'exclude': lambda f: f is None }})
    r"""Certificate reference of the shipment."""
    commercial_invoice: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('commercial_invoice'), 'exclude': lambda f: f is None }})
    contents_explanation: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('contents_explanation'), 'exclude': lambda f: f is None }})
    r"""**required if contents_type is `OTHER`**<br>
    Explanation of the type of goods of the shipment.
    """
    disclaimer: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('disclaimer'), 'exclude': lambda f: f is None }})
    r"""Disclaimer for the shipment and customs information that have been provided."""
    duties_payor: Optional[CustomsDeclarationDutiesPayor] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('duties_payor'), 'exclude': lambda f: f is None }})
    r"""Specifies who will pay the duties for the shipment. Only accepted for FedEx shipments."""
    exporter_identification: Optional[CustomsExporterIdentification] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('exporter_identification'), 'exclude': lambda f: f is None }})
    r"""Additional exporter identification that may be required to ship in certain countries"""
    exporter_reference: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('exporter_reference'), 'exclude': lambda f: f is None }})
    r"""Exporter reference of an export shipment."""
    importer_reference: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('importer_reference'), 'exclude': lambda f: f is None }})
    r"""Importer reference of an import shipment."""
    is_vat_collected: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('is_vat_collected'), 'exclude': lambda f: f is None }})
    r"""Indicates whether the shipment's destination VAT has been collected. May be required for some destinations."""
    invoice: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('invoice'), 'exclude': lambda f: f is None }})
    r"""Invoice reference of the shipment."""
    license: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('license'), 'exclude': lambda f: f is None }})
    r"""License reference of the shipment."""
    metadata: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata'), 'exclude': lambda f: f is None }})
    r"""A string of up to 100 characters that can be filled with any additional information you
    want to attach to the object.
    """
    notes: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('notes'), 'exclude': lambda f: f is None }})
    r"""Additional notes to be included in the customs declaration."""
    address_importer: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('address_importer'), 'exclude': lambda f: f is None }})
    r"""Object ID of the Importer address."""
    eel_pfc: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('eel_pfc'), 'exclude': lambda f: f is None }})
    r"""EEL / PFC type of the shipment. For most shipments from the US to CA, `NOEEI_30_36` is applicable; for most
    other shipments from the US, `NOEEI_30_37_a` is applicable.
    Allowed values available <a href=\"#tag/Customs-Declaration-EELPFC\">here</a>
    """
    incoterm: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('incoterm'), 'exclude': lambda f: f is None }})
    r"""The incoterm reference of the shipment. FCA is available for DHL Express and FedEx only.
    eDAP is available for DPD UK only. DAP is available for DHL Express and DPD UK.
    If expecting DAP for other carriers, please use DDU.
    Allowed values available <a href=\"#tag/Customs-Declaration-Incoterm\">here</a>
    """
    invoiced_charges: Optional[CustomsInvoicedCharges] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('invoiced_charges'), 'exclude': lambda f: f is None }})
    r"""Additional invoiced charges to be shown on the Customs Declaration Commercial Invoice."""
    object_created: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_created'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Date and time of object creation."""
    object_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_id'), 'exclude': lambda f: f is None }})
    r"""Unique identifier of the given object."""
    object_owner: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_owner'), 'exclude': lambda f: f is None }})
    r"""Username of the user who created the object."""
    object_state: Optional[ObjectStateEnum] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_state'), 'exclude': lambda f: f is None }})
    r"""Indicates the validity of the enclosing object"""
    object_updated: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('object_updated'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse, 'exclude': lambda f: f is None }})
    r"""Date and time of last object update."""
    test: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('test'), 'exclude': lambda f: f is None }})
    r"""Indicates whether the object has been created in test mode."""
    




"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from .webhookeventtypeenum import WebhookEventTypeEnum
from dataclasses_json import Undefined, dataclass_json
from shippo import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class WebhookPayloadBatch:
    r"""Content of the webhook posted to the external URL"""
    event: Optional[WebhookEventTypeEnum] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('event'), 'exclude': lambda f: f is None }})
    r"""Type of event that triggered the webhook."""
    test: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('test'), 'exclude': lambda f: f is None }})
    r"""Determines whether the webhook is a test webhook or not."""
    data: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('data'), 'exclude': lambda f: f is None }})
    r"""A string containing the batch object ID, of the form 'batch {batchId} (created|processing complete)'."""
    

